import os

from django.conf import settings

BASE_DIR = os.path.dirname(os.path.realpath(__file__))

DEFAULT_SOURCE_AUTH = (
)

DEFAULT_DATABASE_ROUTING = 'default'

SOURCE_AUTH = getattr(settings, 'MEASUREMENTS_SOURCE_AUTH', DEFAULT_SOURCE_AUTH)

DATABASE_ROUNTING = getattr(settings, 'MEASUREMENTS_DATABASE_ROUTING', DEFAULT_DATABASE_ROUTING)



# Generated by Django 2.2.1 on 2019-05-31 09:58

import django.contrib.postgres.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('measurements', '0003_auto_20190529_0513'),
    ]

    operations = [
        migrations.AlterField(
            model_name='serie',
            name='stats_mean',
            field=models.FloatField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='serie',
            name='stats_outliers',
            field=django.contrib.postgres.fields.ArrayField(base_field=models.IntegerField(), blank=True, null=True, size=None),
        ),
        migrations.AlterUniqueTogether(
            name='measure',
            unique_together={('serie', 'timestamp', 'value')},
        ),
    ]



from datetime import datetime, timedelta
import pandas as pd
import numpy as np

from psqlextra.query import ConflictAction
from measurements.models import Measure, Station, Parameter, Sensor, Serie
import colorbrewer
from numpy import array



def get_serie(station, parameter, sensor='unknown', height=None):
    if not isinstance(station, Station):
        station, created = Station.objects.get_or_create(code=station)
    if not isinstance(parameter, Parameter):
        parameter, created = Parameter.objects.get_or_create(code=parameter)
    if not isinstance(sensor, Sensor):
        sensor, created = Sensor.objects.get_or_create(code=sensor)

    serie, created = Serie.objects.get_or_create(station=station,
                                                 parameter=parameter,
                                                 sensor=sensor,
                                                 height=height)
    return serie


def load_serie(data, serie_id):
    df = pd.DataFrame(data)
    df.dropna(inplace=True)
    # check for empty series
    if df.shape[0] == 0:
        return False
    df.reset_index(inplace=True)
    df.columns = ['timestamp', 'value']
    df['serie_id'] = serie_id

    datadict = df.to_dict(orient='record')
    Measure.extra.on_conflict(df.columns.to_list(),
                              ConflictAction.UPDATE).bulk_insert(datadict)
    return True


def strong_float(value):
    if value in [None, '']:
        return None
    else:
        return float(value)


def get_time(time):
    now = datetime.now()
    if time == 'today':
        start_time = now.replace(hour=0, minute=0, second=0, microsecond=0)
        end_time = now.replace(hour=23, minute=59, second=59, microsecond=0)
    elif time == 'yesterday':
        yesterday = now - timedelta(days=1)
        start_time = yesterday.replace(hour=0, minute=0, second=0, microsecond=0)
        end_time = yesterday.replace(hour=23, minute=59, second=59, microsecond=0)
    elif time == '12h':
        start_time = now - timedelta(hours=12)
        end_time = now
    elif time == '24h':
        start_time = now - timedelta(hours=24)
        end_time = now
    elif time == '36h':
        start_time = now - timedelta(hours=36)
        end_time = now
    elif time == '48h':
        start_time = now - timedelta(hours=48)
        end_time = now
    elif time == '72h':
        start_time = now - timedelta(hours=72)
        end_time = now
    elif time == 'week':
        start_time = now - timedelta(weeks=1)
        end_time = now
    elif time == '30g':
        start_time = now - timedelta(days=30)
        end_time = now

    return start_time, end_time


def classify_fc(fc, k=8, zeros=True, cmap='Blues'):
    """Classify in place a FeatureCollection object.

    Arguments:
    fc -- FeatureCollection object
    k -- number of classes
    """
    items = fc['features']
    values = array([float(i['properties']['value']) for i in items])
    if not zeros:
        _values = values[values != 0]
    else:
        _values = values
    #
    if _values[_values != 0].size == 0:
        _values = array([0, 1])
    counts, bins = np.histogram(_values, bins=k)
    # classes = Equal_Interval(_values, k)

    # cmap = get_cmap(cmap, k)
    cmap = getattr(colorbrewer, cmap)[k]

    for i in items:
        value = float(i['properties']['value'])
        bin = np.fmin(np.digitize(value, bins), k)
        # color = cmap(_class)
        color = cmap[bin -1]
        i['properties']['class'] = str(bin -1)
        # i['properties']['color'] = int(color[0] * 255), int(color[1] * 255), int(color[2] * 255), color[3]
        i['properties']['color'] = color + (1.0,)
        i['properties']['color'] = [str(c) for c in i['properties']['color']]
        i['properties']['value'] = str(round(value, 2))



from datetime import datetime, timedelta
import urllib3
urllib3.disable_warnings()
import requests
import pandas as pd
from bs4 import BeautifulSoup

from measurements.sources.base import BaseSource


ARSO_PARAMS = {'znacilna_visina_valov': 'WAVEH',
               'smer_valovanja': 'WAVED',
               'vodostaj': 'SLEV'
               }


class ArsoAPI(BaseSource):
    def __init__(self, tz='UTC'):
        self.baseurl = 'http://www.arso.gov.si/xml/vode/hidro_podatki_zadnji.xml'
        self.tz = tz
        self.data = None
        self.stations = None

    def get_data(self, force=False):
        if self.data is not None and not force:
            return self.data
        else:
            r = requests.get(self.baseurl)
            self.data = BeautifulSoup(r.text, 'xml')
            return self.data

    def get_stations(self):
        if self.stations is not None:
            return self.stations
        else:
            self.stations = {}
            self.get_data()
            for s in self.data.find_all('postaja'):
                code = s.attrs['sifra']
                label = s.merilno_mesto.string
                self.stations[code] = label
            return self.stations

    def get_df(self, code, last=1):
        parameter = 'SLEV'

        stations = self.get_data()

        data = []
        for p in stations.find_all('postaja', sifra=code):
            _time = datetime.strptime(p.datum.string, "%Y-%m-%d %H:%M")
            time = _time - timedelta(hours=1)
            for tag, parameter in ARSO_PARAMS.items():
                # parameter = 'SLEV'
                t = p.find(tag)
                if t is not None and t.string is not None:
                    _value = float(t.string)
                    if parameter == 'SLEV':
                        _value = _value / 100.
                        baselevel = float(p.attrs['kota_0'])
                        value = baselevel + _value
                    else:
                        value = _value
                    try:
                        data.append({
                            "datetime": time,
                            "value": float(value),
                            'parameter': parameter,
                        })
                    except TypeError as e:
                        print(str(e))
        if len(data) == 0:
            self.df = None
            return self.df
        df = pd.DataFrame(data)
        print(df)
        df.set_index('datetime', inplace=True)
        self.df = df.pivot(values='value', columns='parameter')
        self.df.index = self.df.index.tz_localize(self.tz)
        return self.df




from datetime import datetime
import urllib3
urllib3.disable_warnings()
import requests
import pandas as pd
from bs4 import BeautifulSoup

from measurements.sources.base import BaseSource


class IocAPI(BaseSource):
    def __init__(self, tz='UTC'):
        self.baseurl = "http://www.ioc-sealevelmonitoring.org/bgraph.php?code={}&output=tab&period=7"
        self.tz = tz

    def get_df(self, code, last=1):
        parameter = 'SLEV'

        r = requests.get(self.baseurl.format(code))
        soup = BeautifulSoup(r.text, "html.parser")
        table = soup.find("table")
        if table is None:
            return None
        rows = table.find_all('tr')
        data = []
        for row in rows[1:]:  # skip first row
            td = row.find_all('td')
            _time = datetime.strptime(td[0].text, "%Y-%m-%d %H:%M:%S")
            value = float(td[1].text)
            data.append([_time, value])
        df = pd.DataFrame(data, columns=['datetime', parameter])
        df.set_index('datetime', inplace=True)
        df = df.resample('30Min').nearest()
        self.df = df
        self.df.index = self.df.index.tz_localize(self.tz)

        return self.df




# Generated by Django 2.2.1 on 2019-05-28 15:57

from django.db import migrations

def load_sources(apps, schema_editor):
    SourceType = apps.get_model('measurements', 'SourceType')
    SourceType.objects.create(code="pessl")
    SourceType.objects.create(code="mtt")
    SourceType.objects.create(code="elmed")
    SourceType.objects.create(code="davis")

class Migration(migrations.Migration):

    dependencies = [
        ('measurements', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(load_sources)
    ]



from django.core.management.base import BaseCommand

# from meteo.pgutils import load_data
from measurements.settings import SOURCE_AUTH
from measurements.sources.pessl import PesslAPI
from measurements.sources.mtt import MttAPI
from measurements.models import SourceType, Measure

a = MttAPI()
a.get_df(code='T0193')


from psqlextra.query import ConflictAction


ps = SourceType.objects.get(code='pessl')
for s in ps.station_set.all():
    keys = SOURCE_AUTH['pessl'][s.network.code]
    print(keys)
    pesslapi = PesslAPI(keys['public_key'],
                            keys['private_key'])
    df = pesslapi.get_df(s.code, 1000)
    print(df)



a = df[['HC Air temperature|avg']].copy()
a.reset_index(inplace=True)
a.columns = ['timestamp', 'value']
a['serie_id'] = 2

Measure.extra.on_conflict(['serie_id', 'timestamp', 'value'],
                          ConflictAction.UPDATE).bulk_insert(
    a.to_dict(orient='record'))

from measurements.sources.davis import DavisAPI

ds = DavisAPI()
a = ds.get_df('cantineferrari/ferrari1')


from django.conf import settings
from measurements.sources.elmed import ElmedAPI
elmed = ElmedAPI(None, settings.MEASUREMENTS_SOURCE_AUTH['elmed']['ferrari']['private_key'])
df = elmed.get_df(88)

from measurements.sources.ioc import IocAPI
ioc = IocAPI()
ioc.get_df("TR22")

# create IOC stations
from measurements.models import Measure, Station, Parameter, Sensor, Serie, SourceType, Network
IOC_STATIONS = (
    ('Trieste', 'TR22'),
    ('Venice', 'VE19'),
    ('Ancona', 'AN15'),
    ('S. Benedetto Del Tronto', 'SB36'),
    ('Stari Grad', 'stari'),
    ('Vela Luka', 'vela'),
    ('Sobra', 'sobr'),
    ('Otranto', 'OT15'),
    ('Kerkyra, Corfu', 'corf'),
    ('Crotone', 'CR08'),
    ('Le Castella', 'lcst'),
    ('Itea', 'itea'),
    ('Panormos', 'pano'),
    ('Aigio', 'aigi'),
    ('Katakolo', 'kata'),
    # ('Kyparissia', 'kypa'),
)

network, crated = Network.objects.get_or_create(code='ioc', label='IOC')
source_type, created = SourceType.objects.get_or_create(code='ioc')

for slabel, scode in IOC_STATIONS:
    station, created = Station.objects.get_or_create(code=scode, label=slabel)
    station.source=source_type
    station.network = network
    station.save()

import pandas as pd
from measurements.sources.arso import ArsoAPI
from measurements.models import Measure, Station, Parameter, Sensor, Serie, SourceType
arso = ArsoAPI()

ARSO_CODES = ['9350', '9400', '9410', '9420']

network, crated = Network.objects.get_or_create(code='arso', label='ARSO')
source_type, created = SourceType.objects.get_or_create(code='arso')
stations = arso.get_stations()
for scode in ARSO_CODES:
    slabel = stations.get(scode)
    station, created = Station.objects.get_or_create(code=scode, label=slabel)
    station.source=source_type
    station.network = network
    station.save()


from measurements.models import Measure, Station, Parameter, Sensor, Serie, SourceType, Location
from measurements.sources.getit import GetitAPI
getit = GetitAPI()

data = getit.get_df("PTF - Piattaforma Acqua Alta", last=40)

CNR_STATIONS = {"PTF - Piattaforma Acqua Alta": "PTF - Piattaforma Acqua Alta"}
network, crated = Network.objects.get_or_create(code='cnr', label='CNR')
source_type, created = SourceType.objects.get_or_create(code='cnr')
for scode, slabel in CNR_STATIONS.items():
    station, created = Station.objects.get_or_create(code=scode, label=slabel)
    station.source=source_type
    station.network = network
    station.save()

# copy station code to station label
from measurements.models import Measure, Station, Parameter, Sensor, Serie, SourceType, Location
for s in Station.objects.all():
    if s.label is None and s.code is not None:
        print(s)
        s.label = s.code.replace('_', ' ')
        s.save()


# copy station to location
from measurements.models import Measure, Station, Parameter, Sensor, Serie, SourceType, Location
for s in Station.objects.all():
    if s.label is not None and s.location is None:
        print(s)
        l, created = Location.objects.get_or_create(label = s.label)
        s.location = l
        s.save()

# set empty network to rete telemaerografica
network, crated = Network.objects.get_or_create(code='cpsm', label='Rete Telemareografica')
for s in Station.objects.all():
    if s.network is None:
        s.network = network
        s.save()

"""
create view measurements_grafana_view
as SELECT mm."timestamp",
    mm.value,
    mm.value - ms.stats_mean as value_norm,
    ms.id as serie_id,
    ml.label AS location,
    ml.id AS location_id,
    st_x(ml.geo) AS longitude,
    st_y(ml.geo) AS latitude,
    mn.label as network,
    mn.id as network_id,
    mp.code AS parameter,
    mp.id AS parameter_id,
    ms.height
   FROM measurements_measure mm
     LEFT JOIN measurements_serie ms ON mm.serie_id = ms.id
     LEFT JOIN measurements_station mst ON ms.station_id = mst.id
     LEFT JOIN measurements_location ml ON mst.location_id = ml.id
     LEFT JOIN measurements_network mn ON mst.network_id = mn.id
     LEFT JOIN measurements_parameter mp ON ms.parameter_id = mp.id;
grant select on measurements_grafana_view to public;

create view measurements_grafana_wind_view
as 


select 
 timestamp,
 max(case when parameter = 'WSPD' then value end) as wspd,
 max(case when parameter = 'WDIR' then value end) as wdir,
 location
 from measurements_grafana_view
 where parameter in ('WSPD', 'WDIR')
 GROUP BY timestamp, location
  having wspd not null and wdir not null
;


SELECT mm."timestamp",
    mm.value,
    ml.label AS location,
    ml.id AS location_id,
    mp.code AS parameter,
    mp.id AS parameter_id
   FROM measurements_measure mm
     LEFT JOIN measurements_serie ms ON mm.serie_id = ms.id
     LEFT JOIN measurements_station mst ON ms.station_id = mst.id
     LEFT JOIN measurements_location ml ON mst.location_id = ml.id
     LEFT JOIN measurements_network mn ON mst.network_id = mn.id
     LEFT JOIN measurements_parameter mp ON ms.parameter_id = mp.id;
   WHERE mp.label IN ('WSPD', 'WDIR')
   GROUP BY mm."timestamp";

grant select on measurements_grafana_wind_view to public;
"""

# test for parse line protocol
from datetime import datetime
import pandas as pd
import io
from measurements.models import Measure, Station, Parameter, Sensor, Serie, SourceType, Network
TS_NS = 1000000000
dt = datetime.now()
d = dt - datetime(day=1, month=1, year=1970)
sec = int(d.total_seconds())
sec * TS_NS
#
data = """m,station=test,parameter=SLEV value=10 1573750497000000000
m,station=test,parameter=SLEV value=10 1573750498000000000
m,station=test,parameter=SLEV value=10 1573750499000000000
m,station=test,parameter=SLEV value=10 1573750500000000000
"""""
#
data = """
station,parameter,sensor,value,datetime
itea,SLEV,,10,1573754002000000000
itea,SLEV,,11,1573754002000000000
itea,SLEV,,12,1573754002000000000
itea,SLEV,,11,1573754002000000000
"""""

data = """
station,parameter,sensor,value,timestamp
itea,SLEV,,10,2019-11-14T17:00:00+01:00
itea,SLEV,,11,2019-11-14T17:10:00+01:00
itea,SLEV,,12,2019-11-14T17:20:00+01:00
itea,SLEV,,13,2019-11-14T17:30:00+01:00
itea,SLEV,,11,2019-11-14T17:40:00+01:00
"""""

import requests
iwsurl = "http://127.0.0.1:8000/measurements/writecsv"
r = requests.post(url=iwsurl,
                          data=data,
                          headers={'Content-Type': 'application/octet-stream'})

r.text
df.head()

# code to export - import old data
"""
drop table tmp_measures_arso;
-- select * from measurements_measure_view where network ='CNR';
-- create table tmp_measures_arso as select extract(epoch from timestamp) *1000000000  as timestamp, value, location, parameter from measurements_measure_view where network = 'CNR';
create table tmp_measures_arso 
as 
select extract(epoch from timestamp) *1000000000  as timestamp, value, location, parameter, ms.code as station 
from measurements_measure_view 
left join measurements_station ms on (location=ms.label)
where network = 'IOC';

-- Tržaški zaliv (Zarja) 
-- OB Piran (NIB) 9400
--  Koper - kapitanija  9350
-- alter table tmp_measures_arso add column station varchar;
alter table tmp_measures_arso add column sensor varchar;
-- update tmp_measures_arso set station = '9400' where location = 'OB Piran (NIB)';
-- update tmp_measures_arso set station = '9350' where location = 'Koper - kapitanija';
-- update tmp_measures_arso set station = 'PTF - Piattaforma Acqua Alta' where location = 'PTF - Piattaforma Acqua Alta';

COPY tmp_measures_arso(timestamp, value, station, parameter, sensor) TO '/tmp/tmp_measures_arso' DELIMITER ',' CSV HEADER;
"""

import pandas as pd
data = [
    [1, 'WSPD', 'unknown', 'PTF - Piattaforma Acqua Alta'],
    [2, 'WSPD', 'unknown', 'PTF - Piattaforma Acqua Alta'],
    [3, 'WSPD', 'unknown', 'PTF - Piattaforma Acqua Alta'],
    [1, 'WDIR', 'unknown', 'PTF - Piattaforma Acqua Alta']
]
df = pd.DataFrame(data, columns=['value', 'parameter', 'sensor', 'station'])
_df = df[['parameter', 'sensor', 'station']].drop_duplicates()
_


from django.http.response import JsonResponse
from measurements.models import Location
from django.db.models import Sum, Min, Max
from .utils import get_time, classify_fc
from django.apps import apps
from geojson import Feature, FeatureCollection, loads
from .models import Parameter, Serie, Measure, Station


def locations_flatjson(request):
    return JsonResponse(Location.objects.get_flatjson(), safe=False)



def measure_geojson(request):
    metric = request.GET.get('metric', None)
    if metric is None:
        raise Http404("Metric not found")
    return JsonResponse(get_geojsonpg(metric))


def get_geojsonpg(metric):
    par, aggregate, time, cmap = metric.split('.')
    if aggregate == 'sum':
        AggregateMode = Sum
    elif aggregate == 'min':
        AggregateMode = Min
    elif aggregate == 'max':
        AggregateMode = Max

    start_time, end_time = get_time(time)

    parameter = Parameter.objects.get(code=par)

    qs = Measure.objects
    qs = qs.filter(serie__parameter=parameter)
    qs = qs.filter(timestamp__gte=start_time, timestamp__lte=end_time)

    if aggregate == 'last':
        results = qs.order_by('serie__station', '-timestamp').distinct('serie__station').values('serie__station', 'value')
    else:
        results = qs.values('serie__station').annotate(value=AggregateMode(
            'value'))  # .order_by('parameter', 'location', '-datetime').distinct('parameter', 'location')

    items = []

    for r in results:
        s = Station.objects.get(pk=r['serie__station'])
        if s.location is None:
            continue
        geometry = loads(s.location.geo.geojson)
        properties = {'value': r['value'],
                      'location': s.label}
        items.append(Feature(geometry=geometry.copy(), properties=properties))

    fc = FeatureCollection(items)
    classify_fc(fc, cmap=cmap, zeros=True)
    return fc




from django.core.management.base import BaseCommand

# from meteo.pgutils import load_data
from measurements.settings import SOURCE_AUTH
from measurements.sources.getit import GetitAPI, SOS_PARAMS
from measurements.models import SourceType
from measurements.utils import get_serie, load_serie


class Command(BaseCommand):
    help = "Command to import GETIT-VESK sea level data"

    def handle(self, *args, **options):
        ps = SourceType.objects.get(code='cnr')
        for s in ps.station_set.filter(status='active'):
            apiclient = GetitAPI()
            df = apiclient.get_df(s.code, 24)
            if df is None:
                continue
            for k, v in SOS_PARAMS.items():
                print(df.head())
                if v in df.columns:
                    serie = get_serie(s, v)
                    print(df[v])
                    load_serie(df[v].copy(), serie.id)



from django.core.management.base import BaseCommand

# from meteo.pgutils import load_data
from measurements.settings import SOURCE_AUTH
from measurements.sources.ioc import IocAPI
from measurements.models import SourceType
from measurements.utils import get_serie, load_serie


class Command(BaseCommand):
    help = "Command to import IOC sea level data"

    def handle(self, *args, **options):
        ps = SourceType.objects.get(code='ioc')
        for s in ps.station_set.filter(status='active'):
            apiclient = IocAPI()
            df = apiclient.get_df(s.code)

            if df is not None:
                serie = get_serie(s, 'SLEV')
                load_serie(df['SLEV'].copy(), serie.id)



# Generated by Django 2.2.1 on 2019-06-19 14:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('measurements', '0004_auto_20190531_0958'),
    ]

    operations = [
        migrations.AddField(
            model_name='serie',
            name='height',
            field=models.FloatField(blank=True, null=True),
        ),
    ]



import json
from django.shortcuts import render
from django.http import JsonResponse
from django.views.decorators.csrf import csrf_exempt
from .models import Sensor, Station, Parameter, Serie, Measure
from psqlextra.query import ConflictAction
from basicauth.decorators import basic_auth_required
from django.http import HttpResponse
from datetime import datetime
import pytz
import numpy as np
import pandas as pd
import io

from .utils import get_serie

#
# @basic_auth_required(
#     target_test=lambda request: not request.user.is_authenticated
# )

TS_NS = 1000000000 # to convert from nanoseconds to seconds
SERIE_FIELDS = set(['station', 'sensor', 'parameter'])
SERIES_CACHE = {}


@csrf_exempt
def write_csv(request):
    data = request.body.decode().strip()
    df = pd.read_csv(io.StringIO(data),
                     index_col=None,
                     sep=',', header=1,
                     dtype={'station': object})
    df['timestamp'] = pd.to_datetime(df.timestamp)
    if df.timestamp.dt.tz is None:
        df.timestamp = df.timestamp.dt.tz_localize('UTC')
    df.sensor = df.sensor.fillna('unknown')
    # create missing series
    _df = df[['parameter', 'sensor', 'station']].drop_duplicates()
    for r in _df.to_dict(orient='record'):
        get_serie(**r)
    sdf = pd.DataFrame(Serie.objects.values('id', 'parameter__code', 'station__code', 'sensor__code'))
    sdf.rename(columns={'id': 'serie_id',
                        'parameter__code': 'parameter',
                        'station__code': 'station',
                        'sensor__code': 'sensor'}, inplace=True)
    dfm = df.merge(sdf,
                   left_on=['parameter', 'station', 'sensor'],
                   right_on=['parameter', 'station', 'sensor'])
    cnames = ['serie_id', 'timestamp', 'value']
    _df = dfm[cnames].copy()
    for chunk in np.array_split(_df, _df.shape[0] // 1000 + 1):
        datadict = chunk.to_dict(orient='record')
        Measure.extra.on_conflict(cnames,
                                  ConflictAction.UPDATE).bulk_insert(datadict)
    return JsonResponse({"success": True})


@csrf_exempt
def write(request):
    # rjson = json.loads(request.body)
    # _write(rjson)
    # return JsonResponse(rjson, safe=False)
    # return HttpResponse(request.body)
    res = write_data(request.body)
    # split res


    return JsonResponse(res)

def write_data(body):
    datadict = []
    for l in body.decode().strip().split("\n"):
        datadict.append(parse_line(l))
    # stats = _write(data)
    Measure.extra.on_conflict(['timestamp', 'value', 'serie_id'],
                              ConflictAction.UPDATE).bulk_insert(datadict)
    stats = {'updated': -1,
             'created': -1}
    return stats

def parse_line(line):
    # TODO: using regexp
    measure, others = line.split(",", 1)
    _o = others.split(" ")
    if len(_o) == 3:
        tags, fields, _timestamp = _o
        _timestamp = datetime.fromtimestamp(int(_timestamp)/TS_NS, pytz.utc)
    elif len(_o) == 2:
        tags, fields = _o
        _timestamp = datetime.utcnow()
    r = {'m': measure,
            'ts': _timestamp,
            't': dict(x.split('=') for x in tags.split(',')),
            'f': dict(x.split('=') for x in fields.split(','))
            }

    # check for valid serie
    keys = set(r['t'].keys())
    if not 'serie' in keys and not SERIE_FIELDS <= keys:
        raise ValueError("Incomplete serie definition")

    tags = r['t']
    fields = r['f']
    _station = tags['station']
    _parameter = tags['parameter']
    _sensor = tags.get('sensor', 'unknown')
    skey = frozenset([_station, _parameter, _sensor])
    if skey in SERIES_CACHE.keys():
        serie = SERIES_CACHE[skey]
    else:
        serie = get_serie(_station, _parameter, _sensor)
        SERIES_CACHE[skey] = serie

    _r = {'timestamp': _timestamp,
          'value': fields.get('value'),
          'serie_id': serie.id
    }



    return _r


def _write(rjson):
    stats = {'updated': 0,
             'created': 0}
    for r in rjson:
        tags = r['t']
        fields = r['f']


        _station = tags['station']
        _parameter = tags['parameter']
        _sensor = tags.get('sensor', 'unknown')

        serie = get_serie(_station, _parameter, _sensor)

        timestamp = r.get('ts')
        value = fields.get('value')
        measure, created = Measure.objects.get_or_create(serie=serie,
                                                         timestamp=timestamp,
                                                         value=value)
        if created:
            stats['created'] += 1
        else:
            stats['updated'] += 1
    return stats



# Generated by Django 2.2.1 on 2019-05-29 05:13

from django.db import migrations

def load_unknown_sensor(apps, schema_editor):
    Sensor = apps.get_model('measurements', 'Sensor')
    Sensor.objects.create(code="unknown", label="Unknown sernsor")

class Migration(migrations.Migration):

    dependencies = [
        ('measurements', '0002_auto_20190528_1557'),
    ]

    operations = [
        migrations.RunPython(load_unknown_sensor)
    ]



from datetime import datetime
import urllib3

urllib3.disable_warnings()
import requests
import pandas as pd
from bs4 import BeautifulSoup

from measurements.sources.base import BaseSource
import requests
from bs4 import BeautifulSoup
from datetime import datetime, timedelta
import pandas as pd
from owslib.sos import SensorObservationService
from owslib.fes import FilterCapabilities
from owslib.ows import OperationsMetadata
from owslib.crs import Crs
from owslib.swe.sensor.sml import SensorML
from owslib.etree import etree
from owslib.swe.observation.sos200 import namespaces
from owslib.util import nspath_eval
import json

SOSURL = "http://vesk.ve.ismar.cnr.it/observations/sos/kvp?"
SOSVERSION = "2.0.0"
SOS_PARAMS = {'http://vocab.nerc.ac.uk/collection/P01/current/GHMZAD01/': 'WAVEH'}


class GetitAPI(BaseSource):
    def __init__(self, tz='UTC'):
        self.baseurl = "http://vesk.ve.ismar.cnr.it/observations/sos/kvp?"
        self.tz = tz
        self._service = None
        self._sensors = None
        self._offerings = None
        self._observed_properties = None
        self.data = None

    @property
    def service(self):
        if self._service is None:
            service = SensorObservationService(SOSURL,
                                               version=SOSVERSION)
            self._service = service
        return self._service

    @property
    def sensors(self):
        if self._sensors is None:
            sensors = {}
            get_obs = self.service.get_operation_by_name('GetObservation')
            describe_sensor = self.service.get_operation_by_name('DescribeSensor')
            sensor_ids = describe_sensor.parameters['procedure']['values']
            for sensor_id in sensor_ids:
                response = self.service.describe_sensor(outputFormat='http://www.opengis.net/sensorML/1.0.1',
                                                        procedure=sensor_id)
                tr = etree.fromstring(response)
                element = tr.find('.//' + nspath_eval('sml:SensorML', namespaces))
                ds = SensorML(element)
                name = ds.members[0].name
                sensors[sensor_id] = name
            self._sensors = sensors
        return self._sensors

    @property
    def offerings(self):
        if self._offerings is None:
            self._offerings = [off.id for off in self.service.offerings]
        return self._offerings

    @property
    def observed_properties(self):
        if self._observed_properties is None:
            self._observed_properties = list(set(op for off in self.service.offerings for op in off.observed_properties))
        return self._observed_properties

    def get_df(self, code, last=20):
        if self.data is None:
            end_time = datetime.now()  # - timedelta(hours=120*10)
            start_time = end_time - timedelta(hours=last)
            temporalFilter = "om:phenomenonTime,{}/{}".format(start_time.isoformat(), end_time.isoformat())
            response_format = 'application/json'
            _namespaces = "xmlns(om,http://www.opengis.net/om/2.0)"

            _observed_properties = list(SOS_PARAMS.keys())
            print(_observed_properties)
            _response = self.service.get_observation(offerings=self.offerings,
                                                responseFormat=response_format,
                                                observedProperties=_observed_properties,
                                                namespaces=_namespaces,
                                                temporalFilter=temporalFilter)
            data = []
            print(_response)
            response = json.loads(_response)
            for obs in response['observations']:
                location = obs['featureOfInterest']['name']['value']
                _parameter = obs['observableProperty']
                parameter = SOS_PARAMS.get(_parameter)
                _datetime = obs['phenomenonTime']
                value = float(obs['result']['value'])
                rjson = {
                    'location': location,
                    'parameter': parameter,
                    "datetime": _datetime,
                    "value": value
                }
                data.append(rjson)
            if len(data) == 0:
                return None
            self.data = pd.DataFrame(data)

        _df = self.data[self.data.location == code]
        return _df.pivot(index='datetime', values='value', columns='parameter').head()

def syncvesk():
    sensors = {}
    service = SensorObservationService(SOSURL,
                                       version=SOSVERSION)
    get_obs = service.get_operation_by_name('GetObservation')
    describe_sensor = service.get_operation_by_name('DescribeSensor')
    sensor_ids = describe_sensor.parameters['procedure']['values']
    for sensor_id in sensor_ids:
        response = service.describe_sensor(outputFormat='http://www.opengis.net/sensorML/1.0.1',
                                     procedure=sensor_id)
        tr = etree.fromstring(response)
        element = tr.find('.//' + nspath_eval('sml:SensorML', namespaces))
        ds = SensorML(element)
        name = ds.members[0].name
        sensors[sensor_id] = name


    offerings = [off.id for off in service.offerings]
    _observed_properties = list(set(op for off in service.offerings for op in off.observed_properties))
    observed_properties = SOS_PARAMS.keys()
    end_time = datetime.now() # - timedelta(hours=120*10)
    start_time = end_time - timedelta(hours=12)
    temporalFilter = "om:phenomenonTime,{}/{}".format(start_time.isoformat(), end_time.isoformat())
    response_format = 'application/json'
    _namespaces = "xmlns(om,http://www.opengis.net/om/2.0)"

    _response = service.get_observation(offerings=offerings,
                                        responseFormat=response_format,
                                        observedProperties=observed_properties,
                                        namespaces=_namespaces,
                                        temporalFilter=temporalFilter)
    response = json.loads(_response)
    for obs in response['observations']:
        location = obs['featureOfInterest']['name']['value']
        _parameter = obs['observableProperty']
        parameter = SOS_PARAMS.get(_parameter)
        timestamp = obs['phenomenonTime']
        network = 'CNR'
        value = float(obs['result']['value'])
        rjson = {
            'location': location,
            'parameter': parameter,
            "timestamp": timestamp,
            'network': network,
            "value": value
        }
        _write([rjson])


        pass
    pass



from django.core.management.base import BaseCommand

# from meteo.pgutils import load_data
from measurements.settings import SOURCE_AUTH
from measurements.sources.elmed import ElmedAPI
from measurements.models import SourceType
from measurements.utils import get_serie, load_serie

PARAMETER_MAP = {"temp2m": "AtTemp",
                 # "nasstemp": "At_temp_WetBulb",
                 "relfeuchte": "RelHumidity",
                 "windgeschw": "WindSpeed",
                 "blattnass": "LeafWet",
                 "niederschlag": "Precipitation",
                 "bodentemp25cm": ("SoilTemp", -25),
                 "BF10": ("SoilMoisture", -10),
                 "BF30": ("SoilMoisture", -30),
                 "BF50": ("SoilMoisture", -50),
                 "BF80": ("SoilMoisture", -80),
                 }

class Command(BaseCommand):
    help = "Command for importing ELMED data"

    def handle(self, *args, **options):
        ps = SourceType.objects.get(code='elmed')
        for s in ps.station_set.filter(status='active'):
            self.stdout.write("Loading station {} ... ".format(s), ending='')
            keys = SOURCE_AUTH['elmed'][s.network.code]
            elmedapi = ElmedAPI(None,
                                keys['private_key'])
            df = elmedapi.get_df(s.code, 5)
            if df is not None and df.shape[0] > 0:
                for k, _v in PARAMETER_MAP.items():
                    if not isinstance(_v, str):
                        v, height = _v
                    else:
                        v = _v
                        height = None
                    if k in df.columns:
                        serie = get_serie(s, v, height=height)
                        load_serie(df[k].copy(), serie.id)
                self.stdout.write("[OK]")
            else:
                self.stdout.write("[FAILED]")
